# Simple cell damage calculation example

This example provides a small damage calculation for a scenario earthquake. The example is limited to one 10-arcsec cell.  

## Run the example

Please refer to [README.md](../../README.md) for information on the program requirements and how to install them.  

To run the example calculation, please execute the script  

`run_example_cell.sh`.  

Use   

`python3 damage_calculator.py --help`   

for information on how to use the program input options.   

## Input files  
 
`exposure.csv`: Represents the exposure model with each row providing information for one asset. Each asset in the exposure model is represented by its location, a taxonomy string describing the structural properties of the asset and the number of buildings representing the asset.
 
`ground_motion_values.csv`: Contains the ground-motion values for the example area.

`taxonomy_mapping.csv`: Contains the mapping between the taxonomies of the exposure assets and the names of the fragility functions. The mapping is courtesy of Dr. Helen Crowley (Eucentre, Pavia, Italy).  

`intensity_measure_map.csv`: Contains the mapping between the ground-motion types as named in the ground-motion file header and in the fragility functions.  

`fragilities`: This XML file contains all fragility functions needed for the taxonomies given in the exposure model, according to the mapping in `taxonomy_mapping.csv`. The fragility functions used in this example have been authored by [L. Martins and V. Silva (2020): Development of a fragility and vulnerability model for global seismic risk analyses, Bulletin of Earthquake Engineering](https://doi.org/10.1007/s10518-020-00885-1). Downloaded from [here](https://github.com/lmartins88/global_fragility_vulnerability) on 13 January 2021.  

## Output

The example script writes the damage calculation into `result.cvs`. This output can be compared with a pre-computed reference output in `reference_result.csv` showing the expected result of the example calculation.  

The tree of the `example_cell` directory structure is shown below:

```
.  
├── ...   
├── example_cell     
│   ├── reference_result.csv  
│   ├── run_example_cell.sh
│   └── data
│       ├── exposure.csv             
│       ├── ground_motion_values.csv             
│       ├── taxonomy_mapping.csv    
│       ├── intensity_measure_map.csv    
│       └── fragilities.xml        

```

